/*
* file CcBaseline.java
*
* Licensed Materials - Property of IBM
* Restricted Materials of IBM
* 
* com.ibm.rational.wvcm.stp.cc.CcBaseline
*
* (C) Copyright IBM Corporation 2004, 2012.  All Rights Reserved. 
* Note to U.S. Government Users Restricted Rights:  Use, duplication or  
* disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
*/

package com.ibm.rational.wvcm.stp.cc;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import javax.wvcm.Feedback;
import javax.wvcm.PropertyNameList.PropertyName;
import javax.wvcm.Resource;
import javax.wvcm.ResourceList;
import javax.wvcm.ResourceList.ResponseIterator;
import javax.wvcm.Workspace;
import javax.wvcm.WvcmException;

import com.ibm.rational.wvcm.stpex.StpExEnumeration;

/**
 * <p>A proxy for a ClearCase UCM baseline.
 * </p>
 * <p>
 * A UCM baseline represents a snapshot of the changes made to a particular
 * component in the context of a particular UCM stream: it is a version of a
 * UCM component, recording a version of each element selected by the stream's
 * configuration. The baseline also records the list of activities in the
 * stream whose change sets contain versions of the component's elements.
 * </p>
 * <p>UCM baseline creation differs from the standard WVCM baseline creation model
 * and cannot currently be performed using the CM API.
 * Instead of checking out a UCM component to create a new UCM baseline, the
 * user performs a <i>make baseline </i> operation in the UCM stream. By default this
 * operation automatically creates a new baseline for every component in that
 * stream's configuration that has been modified in that stream since the last
 * <i>make baseline </i> operation.
 * </p>
 * <p>Every component has an <i>initial </i> (root) baseline, which is analogous to
 * a ClearCase element's <i>/main/0 </i> version in that it is empty, and serves
 * as the root of the baseline graph.
 * </p>
 * <p>An <i>import</i> baseline contains the set of versions labeled with a
 * particular label, and is typically used to import non-UCM versions into 
 * a UCM project.
 * </p>
 */
public interface CcBaseline extends javax.wvcm.Baseline, CcVobResource {
    
    /**
     * @see javax.wvcm.Baseline.AddedActivity
     */
    public interface AddedActivityEx extends AddedActivity {

        /**
         * Get the list of versions in the added activity.
         * @return List of CcVersion proxies
         */
        public ResourceList<CcVersion> getVersions();
                
    }

    /**
     * @see javax.wvcm.Baseline.DeletedActivity
     */
    public interface DeletedActivityEx extends DeletedActivity {

        /**
         * Get the list of versions in the deleted activity.
         * @return List of CcVersion proxies
         */
        public ResourceList<CcVersion> getVersions();
    }

    /**
     * @see javax.wvcm.Baseline.PartiallyAddedActivity
     */
    public interface PartiallyAddedActivityEx extends PartiallyAddedActivity {

        /**
         * Get the list of versions in the partially added activity.
         * @return List of CcVersion proxies
         */
        public ResourceList<CcVersion> getVersions();
    }

    /**
     * @see javax.wvcm.Baseline.PartiallyDeletedActivity
     */
    public interface PartiallyDeletedActivityEx extends PartiallyDeletedActivity {

        /**
         * Get the list of versions in the partially deleted activity.
         * @return List of CcVersion proxies
         */
        public ResourceList<CcVersion> getVersions();
    }

    /**
     * Boolean flags for the doCompareReportEx method.
     */
    enum CompareFlagEx {
        /**
         * Provide differences in terms of activites.
         */
        ACTIVITIES,

        /**
         * Provide differences in terms of baselines.
         */
        BASELINES,

        /**
         * Provide differences in terms of versions.
         */
        VERSIONS;
    }
    
    /**
     * Labeling status of this baseline.
     */
    enum LabelingStatus implements StpExEnumeration {
        /** Unlabeled baseline */
        NOT_LABELED,

        /** Incrementally labeled baseline */
        INCREMENTALLY_LABELED,

        /** Fully labeled baseline */
        FULLY_LABELED,
         
        /** Special case: not labeled but doesn't need to be */
        IS_INITIAL,
        
        /** Baseline of a rootless component */
        NO_VERSIONS_TO_LABEL;
    }

    /**
     * <p>Compare two baselines, ClearCase-style.</p>
     * 
     * @param baseline the baseline being compared to this CcBaseline.
     * @param flags specifies the types of differences to include in the
     *              compare report.
     * @param context optional resource (often CcView) providing context for the
     *                generation of certain properties in the returned report. 
     *                May be <b>null</b>.
     * @param feedback the properties available in the returned proxies.
     * @return a ResponseIterator of CompareReport objects, that enumerate the 
     *         differences between the versions selected by this CcBaseline and 
     *         the baseline argument.
     * @throws WvcmException
     */    
    public ResponseIterator<CompareReport>
    doCompareReportEx(CcBaseline baseline, CompareFlagEx[] flags, Resource context, Feedback feedback)
        throws WvcmException;
    
    /**
     * <p>
     * Compare this baseline with the specified stream.
     * </p>
     * <p>
     * All baselines in the stream are used in the comparison as well
     * as any changes in the stream that are not yet captured in a baseline.
     * </p> 
     * @param stream the stream being compared to this CcBaseline.
     * @param flags specifies the types of differences to include in the
     *              compare report.
     * @param context optional resource (often CcView) providing context for the
     *                generation of certain properties in the returned report. 
     *                May be <b>null</b>.
     * @param feedback the properties available in the returned proxies.
     * @return a ResponseIterator of CompareReport objects that enumerate the 
     *         differences between the versions selected by this CcBaseline and 
     *         the stream argument.
     * @throws WvcmException
     */
    public ResponseIterator<CompareReport>
    doCompareReportEx(CcStream stream, CompareFlagEx[] flags, Resource context, Feedback feedback)
        throws WvcmException;

    /**
     * The UCM component for which this is a baseline.
     */
    PropertyName<CcComponent> COMPONENT = 
        new PropertyName<CcComponent>(PROPERTY_NAMESPACE, "baseline-component");

    /**
     * Get the the {@link #COMPONENT} property of this baseline.
     * @return a client proxy for the UCM component that this baseline is in.
     *           Will never be null.
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    public CcComponent getComponent() throws WvcmException;

    /**
     * Is this baseline an "import" baseline?
     */
    PropertyName<Boolean> IS_IMPORT = 
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "baseline-is-import");

    /**
     * Get the {@link #IS_IMPORT} property of this baseline.
     * @return true if this is an import baseline, else false
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    boolean getIsImport() throws WvcmException;

    /**
     * Is this baseline a composite baseline?
     */
    PropertyName<Boolean> IS_COMPOSITE = 
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "baseline-is-composite");

    /**
     * Get the {@link #IS_COMPOSITE} property of this baseline.
     * @return true if this is a composite baseline, else false
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    boolean getIsComposite() throws WvcmException;
    
    /**
     * Is this baseline the initial baseline for its component?
     */
    PropertyName<Boolean> IS_INITIAL = 
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "baseline-is-initial");
    
    /**
     * Get the {@link #IS_INITIAL} property of this baseline.
     * @return   true if this is the initial baseline for its component, else false
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    boolean getIsInitial() throws WvcmException;

    /** This baseline's labeling status. One of:
     * <br>{@link LabelingStatus#NOT_LABELED}
     * <br>{@link LabelingStatus#INCREMENTALLY_LABELED}
     * <br>{@link LabelingStatus#FULLY_LABELED}
     * <br>{@link LabelingStatus#IS_INITIAL}
     * <br>{@link LabelingStatus#NO_VERSIONS_TO_LABEL}
     */
    PropertyName<LabelingStatus> LABELING_STATUS = 
        new PropertyName<LabelingStatus>(PROPERTY_NAMESPACE, "labeling-status");

    /**
     * Get the {@link #LABELING_STATUS} property of this baseline.
     * @return A {@link LabelingStatus} enumerator indicating the labeling 
     *         status of this baseline.
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    LabelingStatus getLabelingStatus() throws WvcmException;
    
    /**
     * Set the value of this baseline's {@link #LABELING_STATUS} property.
     * @param labelingStatus new labeling status for this baseline.
     * The only valid values are {@link LabelingStatus#INCREMENTALLY_LABELED} (if
     * the current labeling status is {@link LabelingStatus#NOT_LABELED}) or 
     * {@link LabelingStatus#FULLY_LABELED} (if the current labeling status 
     * is {@link LabelingStatus#NOT_LABELED} or 
     * {@link LabelingStatus#INCREMENTALLY_LABELED}).
     * When used on a composite baseline, the specified change is applied recursively 
     * to its member baselines. 
     * Can also be set at creation time to specify the labeling status of 
     * baselines(s) created via the {@link #doMakeCcBaselines(MakeCcBaselinesFlag[], Feedback)}
     * method.
     */
    void setLabelingStatus(LabelingStatus labelingStatus);
    
    /**
     * The promotion level of this baseline.
     * <p>
     * Each project VOB includes an ordered set of promotion levels that
     * can be assigned to baselines to indicate the quality or degree of 
     * completeness of the activities and versions represented by the baseline.
     */
    PropertyName<String> PROMOTION_LEVEL = 
        new PropertyName<String>(PROPERTY_NAMESPACE, "promotion-level");
    
    /**
     * Get the {@link #PROMOTION_LEVEL} property of this baseline.
     * @return the {@link #PROMOTION_LEVEL} of this baseline, as a string.
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    String getPromotionLevel() throws WvcmException;
    
    /**
     * Set the value of this baseline's {@link #PROMOTION_LEVEL} property.
     * @param promotionLevel the promotion level of this baseline.
     */
    void setPromotionLevel(String promotionLevel);

    /** 
     * The stream in which this baseline was created.  Will be null for "import"
     * and "initial" baselines because they are not created in a stream context.
     */
    PropertyName<CcStream> STREAM = 
        new PropertyName<CcStream>(PROPERTY_NAMESPACE, "baseline-stream");

    /**
     * Get the the {@link #STREAM} property of this baseline.
     * @return   a client proxy for the UCM stream in which this baseline was
     *           created.  Will be null if this is an import or initial baseline.
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    public CcStream getStream() throws WvcmException;
    
    /**
     * Set the value of this proxy's {@link #STREAM} property.
     * This property can only be set at baseline creation time for use
     * by the {@link #doMakeCcBaselines(MakeCcBaselinesFlag[], Feedback)}
     * method.
     * @param stream Stream on which the new baseline should be created
     * @throws WvcmException
     */
    public void setStream(CcStream stream) throws WvcmException;
        
    /**
     * The list of sub-baselines of this composite baseline.
     * As a configuration gets
     * large, it is often useful to break it up into a set of smaller
     * configurations that form the logical components of that configuration.
     * The sub-baseline list of a baseline of a configuration captures the set
     * of baselines of the component sub-configurations.
     */
    PropertyName<ResourceList<CcBaseline>> SUBBASELINE_LIST =
      new PropertyName<ResourceList<CcBaseline>>(PROPERTY_NAMESPACE, "subbaseline-list");

    /**
     * Get the {@link #SUBBASELINE_LIST} property.
     * @return list of CcBaseline proxies for this baseline
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    public ResourceList<CcBaseline> getSubbaselineList() throws WvcmException;
    
    /**
     * Flags for the <code>doMakeCcBaselines</code> method.
     */
    enum MakeCcBaselinesFlag {
        /**
         *  Create the baseline(s) regardless of whether modifications have occurred 
         */
        IDENTICAL,
        
        /**
         * Recommend the baseline(s) after creation.
         */
        RECOMMEND;
    }
    
    /** 
     * A <em>write-only</em> property used to specify the activities to be
     * included in a new baseline.
     */
    PropertyName<ResourceList<CcActivity>> ACTIVITY_LIST = 
        new PropertyName<ResourceList<CcActivity>>(PROPERTY_NAMESPACE, "mkbl-activity-list");

    /**
     * Set the value of this proxy's {@link #ACTIVITY_LIST} property.
     * This property can only be set at baseline creation time for use
     * by the {@link #doMakeCcBaselines(MakeCcBaselinesFlag[], Feedback)}
     * method.
     * @param activityList List of activities to be used for baseline creation.
     * @throws WvcmException
     */
    public void setActivityList(ResourceList<CcActivity> activityList) throws WvcmException;
    
    /** 
     * A <em>write-only</em> property used to specify the components for
     * which baselines are created.
     */
    PropertyName<ResourceList<CcComponent>> COMPONENT_LIST = 
            new PropertyName<ResourceList<CcComponent>>(PROPERTY_NAMESPACE, "mkbl-component-list");

    /**
     * Set the value of this proxy's {@link #COMPONENT_LIST} property.
     * This property can only be set at baseline creation time for use
     * by the {@link #doMakeCcBaselines(MakeCcBaselinesFlag[], Feedback)}
     * method.
     * @param componentList List of components to be used for baseline creation.
     * @throws WvcmException
     */
    public void setComponentList(ResourceList<CcComponent> componentList) throws WvcmException;

    /** 
     * A <em>write-only</em> property which specifies the workspace which
     * will be used to create baselines. Baselines are created in the stream 
     * that the workspace is attached to.  Setting this overrides setting
     * of the {@link #STREAM} property.
     */
    PropertyName<Workspace> WORKSPACE = 
            new PropertyName<Workspace>(PROPERTY_NAMESPACE, "mkbl-workspace");

    /**
     * Set the value of this proxy's {@link #WORKSPACE} property.
     * This property can only be set at baseline creation time for use
     * by the {@link #doMakeCcBaselines(MakeCcBaselinesFlag[], Feedback)}
     * method.
     * @param workspace Workspace which will be used to create baselines.
     * @throws WvcmException
     */
    public void setWorkspace(Workspace workspace) throws WvcmException;

    /**
     * <p>
     * Create a new UCM baseline using the name segment of the client-specified 
     * location as the root portion of the baseline name.  This interface
     * does not support the use of baseline naming templates.
     * </p>
     * <p>
     * The {@link #STREAM} property should be used to specify the stream
     * on which the baseline(s) will be created.  It is required unless
     * the {@link #WORKSPACE} property is set.
     * </p>
     * <p>
     * The {@link #WORKSPACE} property can be used to specify the workspace which
     * will be used to create baselines. Baselines are created in the stream 
     * that the workspace is attached to.  Setting this overrides setting
     * of the {@link #STREAM} property.
     * </p>
     * <p>
     * The following two properties are mutually exclusive:
     * <ul>
     * <li>The {@link #COMPONENT_LIST} property is used to specify the components for
     * which baselines are created.</li>
     * <li>The {@link #ACTIVITY_LIST} property is used to specify the activities to be
     * included in a new baseline.</li>
     * </ul>
     * </p>
     * <p>
     * The {@link #LABELING_STATUS} property can be used to specify whether the
     * baseline(s) should be unlabeled, incrementally labeled or full labeled.
     * </p>
     * <p>
     * You may also set the following additional baseline properties prior to
     * calling this method:
     * <bl>
     * <li> {@link #COMMENT} </li>
     * </bl>
     * </p>
     * @param flags Resource-specific creation flags.
     */
    public ResourceList<CcBaseline> doMakeCcBaselines(MakeCcBaselinesFlag[] flags, Feedback feedback) throws WvcmException;

}